import 'dart:developer';

import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:hub/shared/components/molecules/locals/index.dart';
import 'package:hub/shared/components/molecules/menu/presentation/blocs/menu_bloc.dart';
import 'package:hub/shared/components/molecules/modules/index.dart';
import 'package:hub/shared/helpers/storage/base_storage.dart';
import 'package:hub/shared/helpers/storage/storage_helper.dart';
import 'package:hub/shared/utils/dialog_util.dart';
import 'package:rxdart/rxdart.dart';

class LocalsRepositoryImpl implements LocalsRepository {
  final LocalsRemoteDataSource remoteDataSource = LocalsRemoteDataSourceImpl();

  static final license = BehaviorSubject<LicenseStatus>();
  static get stream => license.stream;

  @override
  Future<void> validateLocal(BuildContext context) async {
    return await remoteDataSource.checkLocals(context);
  }

  @override
  Future<bool> update(BuildContext context) async {
    bool response = false;
    try {
      while (!response) {
        response = await remoteDataSource.processLocals(context);
        if (!response) {
          await _handleUpdateError(context);
        }
      }
      await _notifyBlocs(context);
    } catch (e) {
      log('Error updating locals: $e');
      await _handleUpdateError(context);
    }
    return response;
  }

  Future<void> _handleUpdateError(BuildContext context) async {
    await StorageHelper().set(KeychainStorageKey.clientUUID.value, '');
    const String errorMsg = 'Erro ao atualizar locais';
    await DialogUtil.error(context, errorMsg);
  }

  Future<void> _notifyBlocs(BuildContext context) async {
    LocalsRepositoryImpl.license.add(false);
    context.read<LocalProfileBloc>().add(LocalProfileEvent());
    context.read<MenuBloc>().add(MenuEvent());
    LocalsRepositoryImpl.license.add(true);
  }

  @override
  Future<void> unlinkLocal(BuildContext context) {
    return remoteDataSource.detachLocal(context);
  }
}
